// Copyright 2022, domohuhn.
// License: BSD-3-Clause
// See LICENSE for the full text of the license

import 'package:mutation_test/src/reports/report_data.dart';
import 'package:mutation_test/src/version.dart';
import 'package:mutation_test/src/reports/html_report.dart';
import 'package:test/test.dart';

import 'create_test_data.dart';
import '../core/mock_system_interactions.dart';

void main() {
  test('Create top level html file', () {
    var reporter = ReportData(true, MockSystemInteractions());
    reporter.addInputFile('test.xml');
    var result = createTopLevelHtmlFile(reporter);
    // exclude report creation time
    final end1 = emptyTopLevel.indexOf('Date:') + 49;
    final start2 = end1 + 26;
    expect(result.substring(0, end1), emptyTopLevel.substring(0, end1));
    expect(result.substring(start2), emptyTopLevel.substring(start2));
  });

  test('Create html source report', () {
    final reporter = createTestData();
    var result = createSourceHtmlFile(
        reporter, reporter.testedFiles.values.first, 'test.html');
    // exclude report creation time
    final end1 = htmlSourceFileReport.indexOf('Date:') + 49;
    final start2 = end1 + 26;
    expect(result.substring(0, end1), htmlSourceFileReport.substring(0, end1));
    expect(result.substring(start2), htmlSourceFileReport.substring(start2));
  });
}

final emptyTopLevel = '<!DOCTYPE html>\n'
    '<html lang="en">\n'
    '<head>\n'
    '<meta name="viewport" content="width=device-width, initial-scale=1">\n'
    '<style>\n${getCSSFileContents()}'
    '</style>\n'
    '</head>\n'
    '<body>\n'
    '<table width ="100%" cellspacing="0" border="0">\n'
    '    <tr><td class="title">Mutation test report</td></tr>\n'
    '     <tr><td><hr class="ruler"/></td></tr>\n'
    '\n'
    '     <tr>\n'
    '     <td width="100%">\n'
    '     <table width="100%" cellpadding="1" border="0">\n'
    '     <tbody><tr>\n'
    '     <td class="ItemLabel" width="10%">Current display:</td>\n'
    '     <td class="ItemText" width="35%">top level</td>\n'
    '     <td width="10%"></td>\n'
    '     <td class="MiddleHeader" width="15%">Detected</td>\n'
    '     <td class="MiddleHeader" width="15%">Total</td>\n'
    '     <td class="MiddleHeader" width="15%">Percentage</td>\n'
    '     </tr>\n'
    '\n'
    '     <tr>\n'
    '     <td class="ItemLabel" width="10%">Date:</td>\n'
    '     <td class="ItemText" width="35%">2022-10-30 15:40:12.063141</td>\n'
    '     <td class="ItemLabel" width="10%">Mutations:</td>\n'
    '     <td class="ItemReport" width="15%">0</td>\n'
    '     <td class="ItemReport" width="15%">0</td>\n'
    '     <td class="ItemReportHigh" width="15%">100.0 %</td>\n'
    '     </tr>\n'
    '     \n'
    '     <tr>\n'
    '     <td class="ItemLabel" width="10%">Builtin rules:</td>\n'
    '     <td class="ItemText" width="35%">true</td>\n'
    '     <td class="ItemLabel" width="10%">Blocked:</td>\n'
    '     <td class="ItemReport" width="15%">0</td>\n'
    '     <td class="ItemReport" width="15%">0</td>\n'
    '     <td class="ItemReportHigh" width="15%">0.0 %</td>\n'
    '     </tr>\n'
    '     <tr>\n'
    '     <td class="ItemLabel" width="10%">Quality rating:</td>\n'
    '     <td class="ItemText" width="35%">N/A</td>\n'
    '     <td class="ItemLabel" width="10%">Success:</td>\n'
    '     <td class="ItemText" width="15%">true</td>\n'
    '     <td class="ItemText" width="15%"></td>\n'
    '     <td class="ItemText" width="15%"></td>\n'
    '     </tr>\n'
    '       </tbody>\n'
    '     </table>\n'
    '     </td>\n'
    '     </tr>\n'
    '     \n'
    '\n'
    '     \n'
    '     <tr><td><hr class="ruler"/></td></tr>\n'
    '</table>\n'
    '\n'
    '<center>\n'
    '<table width ="80%" cellspacing="1" border="0">\n'
    '     <tbody>\n'
    '     <tr><td width="60%"></td><td width="10%"></td><td width="10%"></td><td width="10%"></td><td width="10%"></td></tr>\n'
    '     <tr><td class="ItemHead" width="60%">Path</td><td class="ItemHead" width="30%" colspan="3">Detection rate</td><td class="ItemHead" width="10%">Blocked</td></tr>\n'
    '    </tbody>\n'
    '</table>\n'
    '</center>\n'
    '\n'
    '\n'
    '<table width ="100%" cellspacing="0" border="0">\n'
    '  <tr><td><hr class="ruler"/></td></tr>\n'
    '  <tr><td class="footer">Generated by ${mutationTestVersion()}</td></tr>\n'
    '</table>\n'
    '</body>\n'
    '</html>\n'
    '';

final htmlSourceFileReport = '<!DOCTYPE html>\n'
    '<html lang="en">\n'
    '<head>\n'
    '<meta name="viewport" content="width=device-width, initial-scale=1">\n'
    '<style>\n${getCSSFileContents()}'
    '</style>\n'
    '</head>\n'
    '<body>\n'
    '<table width ="100%" cellspacing="0" border="0">\n'
    '    <tr><td class="title">Mutation test report</td></tr>\n'
    '     <tr><td><hr class="ruler"/></td></tr>\n'
    '\n'
    '     <tr>\n'
    '     <td width="100%">\n'
    '     <table width="100%" cellpadding="1" border="0">\n'
    '     <tbody><tr>\n'
    '     <td class="ItemLabel" width="10%">Current display:</td>\n'
    '     <td class="ItemText" width="35%">path.dart - <a href="./test.html">back to top</a></td>\n'
    '     <td width="10%"></td>\n'
    '     <td class="MiddleHeader" width="15%">Detected</td>\n'
    '     <td class="MiddleHeader" width="15%">Total</td>\n'
    '     <td class="MiddleHeader" width="15%">Percentage</td>\n'
    '     </tr>\n'
    '\n'
    '     <tr>\n'
    '     <td class="ItemLabel" width="10%">Date:</td>\n'
    '     <td class="ItemText" width="35%">2022-10-30 16:37:03.526343</td>\n'
    '     <td class="ItemLabel" width="10%">Mutations:</td>\n'
    '     <td class="ItemReport" width="15%">1</td>\n'
    '     <td class="ItemReport" width="15%">3</td>\n'
    '     <td class="ItemReportLow" width="15%">33.3 %</td>\n'
    '     </tr>\n'
    '     \n'
    '     <tr>\n'
    '     <td class="ItemLabel" width="10%">Builtin rules:</td>\n'
    '     <td class="ItemText" width="35%">true</td>\n'
    '     <td class="ItemLabel" width="10%">Blocked:</td>\n'
    '     <td class="ItemReport" width="15%">1</td>\n'
    '     <td class="ItemReport" width="15%">3</td>\n'
    '     <td class="ItemReportMedium" width="15%">33.3 %</td>\n'
    '     </tr>\n'
    '     </tbody>\n'
    '     </table>\n'
    '     </td>\n'
    '     </tr>\n'
    '     \n'
    '\n'
    '     \n'
    '     <tr><td><hr class="ruler"/></td></tr>\n'
    '</table>\n'
    '\n'
    '<pre class="fileHeader">          Source code</pre>\n'
    '<pre class="fileContents">\n'
    '<a name="1"><button class="collapsible problem"><pre class="fileContents"><span class="lineNumber">       1 </span>var x = 0;</pre></button>\n'
    '<div class="content">\n'
    '<b>Undetected mutations:</b>\n'
    '<table class="mutationTable" width="100%">\n'
    '<tr><td class="mutationLabel" width="5%">1 :</td><td class="mutationText" width="87%"><span class="addedLine">+ <span class="changedTokens">var x</span> = a;</span></td><td class="match">[0-9]+</td></tr></table>\n'
    '<b>Detected mutations:</b>\n'
    '<table class="mutationTable" width="100%">\n'
    '<tr><td class="mutationLabel" width="5%">1 :</td><td class="mutationText" width="87%"><span class="addedLine">+ <span class="changedTokens">var x </span>= -0;</span></td><td class="match">[0-9]+</td></tr></table>\n'
    '<b>Mutations that caused a time out:</b>\n'
    '<table class="mutationTable" width="100%">\n'
    '<tr><td class="mutationLabel" width="5%">1 :</td><td class="mutationText" width="87%"><span class="addedLine">+ <span class="changedTokens">var x</span> = c;</span></td><td class="match">[0-9]+<span class="tooltip">Id: testId</span></td></tr></table>\n'
    '\n'
    '</div></a><a name="2"><span class="lineNumber">       2 </span></a>\n'
    '<a name="3"><span class="lineNumber">       3 </span>// mooo</a>\n'
    '<a name="4"><span class="lineNumber">       4 </span></a>\n'
    '</pre>\n'
    '<script>\n'
    'var coll = document.getElementsByClassName("collapsible");\n'
    'var i;\n'
    '\n'
    'for (i = 0; i < coll.length; i++) {\n'
    '  coll[i].addEventListener("click", function() {\n'
    '    this.classList.toggle("active");\n'
    '    var content = this.nextElementSibling;\n'
    '    if (content.style.maxHeight){\n'
    '      content.style.maxHeight = null;\n'
    '    } else {\n'
    '      content.style.maxHeight = content.scrollHeight + "px";\n'
    '    }\n'
    '\tfor (k = 0; k < coll.length; k++) {\n'
    '      var content2 = coll[k].nextElementSibling;\n'
    '      if (content2.style.maxHeight && content.parentElement == content2){\n'
    '      content2.style.maxHeight = content2.scrollHeight + content.scrollHeight + "px";\n'
    '    }\n'
    '      \n'
    '    }\n'
    '    \n'
    '  });\n'
    '}\n'
    '</script><table width ="100%" cellspacing="0" border="0">\n'
    '  <tr><td><hr class="ruler"/></td></tr>\n'
    '  <tr><td class="footer">Generated by ${mutationTestVersion()}</td></tr>\n'
    '</table>\n'
    '</body>\n'
    '</html>\n'
    '';
